package de.hft.stuttgart.citydoctor2.datastructure;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.citygml4j.core.model.core.AbstractGenericAttributeProperty;
import org.citygml4j.core.model.generics.*;


public class GenericAttribute {

    private static final Logger logger = LogManager.getLogger(GenericAttribute.class);

    private final AbstractGenericAttributeProperty original;

    private String type;
    private String name;
    private String value;

    public GenericAttribute(AbstractGenericAttributeProperty attributeProperty) {
        original = attributeProperty;
        if (attributeProperty.getObject() == null) {
            logger.warn("GenericAttribute {} is null", attributeProperty);
            return;
        }
        name = attributeProperty.getObject().getName();

        if (attributeProperty.getObject() instanceof StringAttribute sa) {
            type = "StringAttribute";
            value = String.format("\"%s\"", sa.getValue());
        } else if (attributeProperty.getObject() instanceof IntAttribute ia) {
            type = "IntAttribute";
            value = ia.getValue().toString();
        } else if (attributeProperty.getObject() instanceof DoubleAttribute da) {
            type = "DoubleAttribute";
            value = da.getValue().toString();
        } else if (attributeProperty.getObject() instanceof DateAttribute date) {
            type = "DateAttribute";
            value = date.getValue().toString();
        } else if (attributeProperty.getObject() instanceof UriAttribute ua) {
            type = "UriAttribute";
            value = ua.getValue();
        } else if (attributeProperty.getObject() instanceof MeasureAttribute ma) {
            type = "MeasureAttribute";
            value = String.format("%s %s", ma.getValue().getValue().toString(), ma.getValue().getUom());
        } else if (attributeProperty.getObject() instanceof CodeAttribute ca) {
            type = String.format("CodeAttribute (%s)", ca.getValue().getLanguage());
            value = String.format("'''%s''' %n CodeSpace: %s", ca.getValue().getValue(), ca.getValue().getCodeSpace());
        } else {
            logger.warn("GenericAttribute {} is of unknown type {}", attributeProperty, attributeProperty.getObject());
            value = attributeProperty.getObject().getValue().toString();
            type = "Unknown type";
        }

    }

    public AbstractGenericAttributeProperty getOriginal() {
        return original;
    }

    public String getName() {
        return name;
    }

    public String getValue() {
        return value;
    }

    @Override
    public String toString() {
        return String.format("%s: [ %s = %s ]", type, name, value);
    }
}
